/* -*- Mode: indent-tabs-mode: nil; js-indent-level: 2 -*- */
/* vim: set sts=2 sw=2 et tw=80: */
"use strict";

const inIDOMUtils = Cc["@mozilla.org/inspector/dom-utils;1"].getService(Ci.inIDOMUtils);

add_task(function* testExtremeThemeProperties() {
  let tab = yield BrowserTestUtils.openNewForegroundTab(gBrowser, "about:home", true);

  let manifest = {
    manifest: {
      "theme": {
        "colors": {
          "background_tab": [76, 158, 217], // #4c9ed9
          "background_tab_inactive": [39, 43, 53], // #272b35
          "background_tab_text": [245, 247, 250], // #F5F7FA
          "button_background": [37, 44, 51], // #252C33
          "frame": [39, 43, 53], // #272b35
          "frame_inactive": [57, 63, 76], // #393F4C
          "ntp_background": [255, 170, 255],
          "ntp_text": [0, 85, 187],
          "ntp_link": [170, 255, 255],
          "ntp_link_underline": [0, 85, 187],
          "ntp_header": [0, 85, 187],
          "ntp_section": [0, 85, 187],
          "ntp_section_text": [0, 85, 187],
          "ntp_section_link": [0, 85, 187],
          "ntp_section_link_underline": [0, 85, 187],
          "tab_text": [245, 247, 250], // #f5f7fa
          "toolbar": [39, 43, 53], // #272b35
          "toolbar_bottom_separator": [0, 0, 0, 0.2], // rgba(0,0,0,.2)
          "toolbar_button_stroke": [25, 33, 38, 0.8], // rgba(25,33,38,.8)
          "toolbar_button_stroke_inactive": [25, 33, 38, 0.6], // rgba(25,33,38,.6)
          "toolbar_input_control": [23, 27, 31], // #171B1F
          "toolbar_input_control_stroke": [29, 35, 40], // #1D2328
          "toolbar_top_separator": [0, 0, 0, 0.2], // rgba(0,0,0,.2)
          "toolbar_vertical_separator": [95, 102, 112], // #5F6670
        },
        "tints": {
          "background_tab": [0.56, 0.18, 0.04], // #07090a
          "buttons": [0.64, 0.56, 0.22], // rgb(25,33,38)
        },
        "gradients": {
          "toolbar_button": "rgba(25,33, 38,.6) linear-gradient(rgba(25,33,38,.6), rgba(25,33,38,.6)) padding-box",
          "toolbar_button_pressed": "rgba(25,33,38,1) linear-gradient(rgba(25,33,38,1), rgba(25,33,38,1)) border-box",
        },
        "properties": {
          "navbar_padding": 0,
          "space_above_tabbar": 0,
          "square_tabs": true,
          "square_backbutton": true,
          "toolbar_button_shadow": "none",
          "toolbar_button_shadow_inactive": "none",
          "toolbar_navbar_highlight_overlap": 0,
          "toolbar_navbar_overlap": 0,
          "toolbar_text_shadow": "none",
        },
      },
    },
    background() {
      browser.test.onMessage.addListener((message) => {
        if (message !== "update-theme") {
          return;
        }

        const kGradient = ["#ffaaff linear-gradient(", "deg, #ffaaff, #aaffff)"];
        const kTimeoutMs = 100;

        let gDegrees = 135;

        browser.theme.update({
          "colors": {
            "background_tab": [255, 255, 255, 0.6],
            "background_tab_inactive": [255, 255, 255, 0.2],
            "background_tab_text": [0, 85, 187],
            "button_background": [255, 255, 255, 0.05],
            "frame": kGradient.join(gDegrees),
            "frame_inactive": [238, 68, 255, 0.1],
            "tab_text": [0, 85, 187], // #0055bb
            "toolbar": [255, 255, 255, .05],
            "toolbar_bottom_separator": [238, 68, 255, .2], // #ee44ff
            "toolbar_button_stroke": [238, 68, 255, .8], // #ee44ff
            "toolbar_button_stroke_inactive": [238, 68, 255, .6], // #ee44ff
            "toolbar_input_control": [255, 255, 255, .95],
            "toolbar_input_control_stroke": [238, 68, 255], // #ee44ff
            "toolbar_top_separator": [255, 255, 255, 0.2],
            "toolbar_vertical_separator": [238, 68, 255], // #ee44ff
          },
          "tints": {
            "background_tab": [0, 0, 1, 0.25],
            "buttons": [0, 0, 1, 0.05],
          },
          "gradients": {
            "toolbar_button": "none",
            "toolbar_button_pressed": "none",
          },
        });

        setTimeout(() => {
          gDegrees++;
          browser.theme.update({
            colors: {frame: kGradient.join(gDegrees)},
          });

          browser.test.sendMessage("theme-updated");
        }, kTimeoutMs);
      });
    },
  };
  let extension = ExtensionTestUtils.loadExtension(manifest);

  yield extension.startup();

  let document = tab.ownerDocument;
  let window = document.defaultView;

  let style = window.getComputedStyle(document.documentElement);

  Assert.ok(style.backgroundImage, "Expected background image");
  Assert.equal(style.color, "rgb(245, 247, 250)", "Expected correct text color");

  extension.sendMessage("update-theme");
  yield extension.awaitMessage("theme-updated");

  style = window.getComputedStyle(document.documentElement);
  // HALP! The following style fetch keeps returning the old, initial value. Why?
  // Is this a bug?
  // Assert.equal(style.backgroundImage, "linear-gradient(136deg, #ffaaff, #aaffff)",
  //   "Expected correct background image");
  Assert.equal(style.color, "rgb(0, 85, 187)", "Expected correct text color");

  // Pick a button to test updated colors.
  let button = document.getElementById("PanelUI-menu-button");
  inIDOMUtils.addPseudoClassLock(button, ":hover");
  style = window.getComputedStyle(button);
  Assert.equal(style.backgroundImage, "none", "Expected explicitly empty background");
  inIDOMUtils.clearPseudoClassLocks(button);

  yield extension.unload();

  yield BrowserTestUtils.removeTab(tab);
});
